import {Store} from "./store";
import {SDKParams} from "./sdkParams";
import ChatEvents from "../events.module";
import MessageQueues from "./store/messageQueues";
import HTTPRequestsManager from "./helpers/httpRequest";
import ErrorHandler from "./errorHandler";
import DOMPurify from "dompurify";
import {JSDOM} from "jsdom";
import FS from 'fs';
import FileMethods from "./chat/fileMethods";

function App() {

    const app = {};
    app.errorHandler = new ErrorHandler(app);
    app.store = new Store(app);
    app.sdkParams = new SDKParams();
    app.chatEvents = new ChatEvents(app);
    app.messageQueues = new MessageQueues(app);
    app.httpRequest = new HTTPRequestsManager(app);
    app.domPurify = DOMPurify(new JSDOM('').window);
    app.fs =  FS;
    app.fileMethods = new FileMethods(app);

    return app;
}

export default App;